@@ -16,16 +16,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-import chess
-import collections
+import collections.abc
 import itertools
-import re
 import logging
+import re
+import weakref
 
-try:
-    from collections.abc import MutableMapping  # Python 3
-except ImportError:
-    from collections import MutableMapping  # Python 2
+import chess
 
 
 LOGGER = logging.getLogger(__name__)
@@ -109,8 +106,10 @@ MOVETEXT_REGEX = re.compile(r"""
 TAG_ROSTER = ["Event", "Site", "Date", "Round", "White", "Black", "Result"]
 
 
-class GameNode(object):
+SKIP = object()
 
+
+class GameNode:
     def __init__(self):
         self.parent = None
         self.move = None
@@ -121,20 +120,22 @@ class GameNode(object):
 
         self.board_cached = None
 
-    def board(self, _cache=True):
+    def board(self, *, _cache=True):
         """
         Gets a board with the position of the node.
 
         It's a copy, so modifying the board will not alter the game.
         """
-        if self.board_cached:
-            return self.board_cached.copy()
+        if self.board_cached is not None:
+            board = self.board_cached()
+            if board is not None:
+                return board.copy()
 
         board = self.parent.board(_cache=False)
         board.push(self.move)
 
         if _cache:
-            self.board_cached = board
+            self.board_cached = weakref.ref(board)
             return board.copy()
         else:
             return board
@@ -148,7 +149,7 @@ class GameNode(object):
         """
         return self.parent.board().san(self.move)
 
-    def uci(self, chess960=None):
+    def uci(self, *, chess960=None):
         """
         Gets the UCI notation of the move leading to this node.
         See :func:`chess.Board.uci()`.
@@ -158,7 +159,7 @@ class GameNode(object):
         return self.parent.board().uci(self.move, chess960=chess960)
 
     def root(self):
-        """Gets the root node, i.e. the game."""
+        """Gets the root node, i.e., the game."""
         node = self
 
         while node.parent:
@@ -185,7 +186,7 @@ class GameNode(object):
         comment). The root node does not start a variation and can have no
         starting comment.
 
-        For example in ``1. e4 e5 (1... c5 2. Nf3) 2. Nf3`` the node holding
+        For example, in ``1. e4 e5 (1... c5 2. Nf3) 2. Nf3``, the node holding
         1... c5 starts a variation.
         """
         if not self.parent or not self.parent.variations:
@@ -193,8 +194,8 @@ class GameNode(object):
 
         return self.parent.variations[0] != self
 
-    def is_main_line(self):
-        """Checks if the node is in the main line of the game."""
+    def is_mainline(self):
+        """Checks if the node is in the mainline of the game."""
         node = self
 
         while node.parent:
@@ -207,6 +208,9 @@ class GameNode(object):
 
         return True
 
+    # TODO: Deprecated
+    is_main_line = is_mainline
+
     def is_main_variation(self):
         """
         Checks if this node is the first variation from the point of view of its
@@ -217,15 +221,21 @@ class GameNode(object):
 
         return not self.parent.variations or self.parent.variations[0] == self
 
+    def __getitem__(self, move):
+        try:
+            return self.variations[move]
+        except TypeError:
+            for variation in self.variations:
+                if variation.move == move or variation == move:
+                    return variation
+
+        raise KeyError(move)
+
     def variation(self, move):
         """
         Gets a child node by either the move or the variation index.
         """
-        for index, variation in enumerate(self.variations):
-            if move == variation.move or index == move or move == variation:
-                return variation
-
-        raise KeyError("variation not found")
+        return self[move]
 
     def has_variation(self, move):
         """Checks if the given *move* appears as a variation."""
@@ -233,20 +243,20 @@ class GameNode(object):
 
     def promote_to_main(self, move):
         """Promotes the given *move* to the main variation."""
-        variation = self.variation(move)
+        variation = self[move]
         self.variations.remove(variation)
         self.variations.insert(0, variation)
 
     def promote(self, move):
         """Moves a variation one up in the list of variations."""
-        variation = self.variation(move)
+        variation = self[move]
         i = self.variations.index(variation)
         if i > 0:
             self.variations[i - 1], self.variations[i] = self.variations[i], self.variations[i - 1]
 
     def demote(self, move):
         """Moves a variation one down in the list of variations."""
-        variation = self.variation(move)
+        variation = self[move]
         i = self.variations.index(variation)
         if i < len(self.variations) - 1:
             self.variations[i + 1], self.variations[i] = self.variations[i], self.variations[i + 1]
@@ -255,7 +265,7 @@ class GameNode(object):
         """Removes a variation."""
         self.variations.remove(self.variation(move))
 
-    def add_variation(self, move, comment="", starting_comment="", nags=()):
+    def add_variation(self, move, *, comment="", starting_comment="", nags=()):
         """Creates a child node with the given attributes."""
         node = GameNode()
         node.move = move
@@ -266,7 +276,7 @@ class GameNode(object):
         self.variations.append(node)
         return node
 
-    def add_main_variation(self, move, comment=""):
+    def add_main_variation(self, move, *, comment=""):
         """
         Creates a child node with the given attributes and promotes it to the
         main variation.
@@ -276,14 +286,18 @@ class GameNode(object):
         self.variations.insert(0, node)
         return node
 
-    def main_line(self):
-        """Yields the moves of the main line starting in this node."""
-        node = self
-        while node.variations:
-            node = node.variations[0]
-            yield node.move
+    def mainline(self):
+        """Returns an iterator over the mainline starting after this node."""
+        return Mainline(self)
 
-    def add_line(self, moves, comment="", starting_comment="", nags=()):
+    def mainline_moves(self):
+        """Returns an iterator over the main moves after this node."""
+        return Mainline(self, lambda node: node.move)
+
+    # TODO: Deprecated
+    main_line = mainline_moves
+
+    def add_line(self, moves, *, comment="", starting_comment="", nags=()):
         """
         Creates a sequence of child nodes for the given list of moves.
         Adds *comment* and *nags* to the last node of the line and returns it.
@@ -305,95 +319,84 @@ class GameNode(object):
 
         return node
 
-    def accept(self, visitor, _board=None):
+    def _accept_node(self, parent_board, visitor):
+        if self.starting_comment:
+            visitor.visit_comment(self.starting_comment)
+
+        visitor.visit_move(parent_board, self.move)
+
+        for nag in sorted(self.nags):
+            visitor.visit_nag(nag)
+
+        if self.comment:
+            visitor.visit_comment(self.comment)
+
+    def accept(self, visitor, *, _parent_board=None):
         """
-        Traverse game nodes in PGN order using the given *visitor*. Returns
-        the visitor result.
+        Traverses game nodes in PGN order using the given *visitor*. Starts with
+        the move leading to this node. Returns the *visitor* result.
         """
-        board = self.board() if _board is None else _board
+        board = self.parent.board() if _parent_board is None else _parent_board
 
-        # The first move of the main line goes first.
+        # First, visit the move that leads to this node.
+        self._accept_node(board, visitor)
+
+        # Then visit sidelines.
+        if _parent_board is not None and self == self.parent.variations[0]:
+            for variation in itertools.islice(self.parent.variations, 1, None):
+                if visitor.begin_variation() is not SKIP:
+                    variation.accept(visitor, _parent_board=board)
+                visitor.end_variation()
+
+        # The mainline is continued last.
         if self.variations:
-            main_variation = self.variations[0]
-            visitor.visit_move(board, main_variation.move)
-
-            # Visit NAGs.
-            for nag in sorted(main_variation.nags):
-                visitor.visit_nag(nag)
-
-            # Visit the comment.
-            if main_variation.comment:
-                visitor.visit_comment(main_variation.comment)
-
-        # Then visit side lines.
-        for variation in itertools.islice(self.variations, 1, None):
-            # Start variation.
-            visitor.begin_variation()
-
-            # Append starting comment.
-            if variation.starting_comment:
-                visitor.visit_comment(variation.starting_comment)
-
-            # Visit move.
-            visitor.visit_move(board, variation.move)
-
-            # Visit NAGs.
-            for nag in sorted(variation.nags):
-                visitor.visit_nag(nag)
-
-            # Visit comment.
-            if variation.comment:
-                visitor.visit_comment(variation.comment)
-
-            # Recursively append the next moves.
-            board.push(variation.move)
-            variation.accept(visitor, _board=board)
-            board.pop()
-
-            # End variation.
-            visitor.end_variation()
-
-        # The main line is continued last.
-        if self.variations:
-            main_variation = self.variations[0]
-
-            # Recursively append the next moves.
-            board.push(main_variation.move)
-            main_variation.accept(visitor, _board=board)
+            board.push(self.move)
+            self.variations[0].accept(visitor, _parent_board=board)
             board.pop()
 
         # Get the result if not called recursively.
-        if _board is None:
+        if _parent_board is None:
             return visitor.result()
 
     def accept_subgame(self, visitor):
         """
         Traverses headers and game nodes in PGN order, as if the game was
-        starting from this node. Returns the visitor result.
+        starting after this node. Returns the *visitor* result.
         """
-        game = self.root()
-        visitor.begin_game()
+        if visitor.begin_game() is not SKIP:
+            game = self.root()
+            board = self.board()
 
-        dummy_game = Game.without_tag_roster()
-        dummy_game.setup(self.board())
+            dummy_game = Game.without_tag_roster()
+            dummy_game.setup(board)
 
-        visitor.begin_headers()
-        for tagname, tagvalue in game.headers.items():
-            if tagname not in dummy_game.headers:
+            visitor.begin_headers()
+
+            for tagname, tagvalue in game.headers.items():
+                if tagname not in dummy_game.headers:
+                    visitor.visit_header(tagname, tagvalue)
+            for tagname, tagvalue in dummy_game.headers.items():
                 visitor.visit_header(tagname, tagvalue)
-        for tagname, tagvalue in dummy_game.headers.items():
-            visitor.visit_header(tagname, tagvalue)
-        visitor.end_headers()
 
-        self.accept(visitor)
+            if visitor.end_headers() is not SKIP:
+                if self.variations:
+                    self.variations[0].accept(visitor, _parent_board=board)
+
+                visitor.visit_result(game.headers.get("Result", "*"))
 
-        visitor.visit_result(game.headers.get("Result", "*"))
         visitor.end_game()
         return visitor.result()
 
     def __str__(self):
         return self.accept(StringExporter(columns=None))
 
+    def __repr__(self):
+        return "<GameNode at {} ({}{} {} ...)>".format(
+            hex(id(self)),
+            self.parent.board().fullmove_number,
+            "." if self.parent.board().turn == chess.WHITE else "...",
+            self.san())
+
 
 class Game(GameNode):
     """
@@ -403,42 +406,22 @@ class Game(GameNode):
     """
 
     def __init__(self, headers=None):
-        super(Game, self).__init__()
+        super().__init__()
         self.headers = Headers(headers)
         self.errors = []
 
-    def board(self, _cache=False):
+    def board(self, *, _cache=False):
         """
         Gets the starting position of the game.
 
         Unless the ``FEN`` header tag is set, this is the default starting
         position (for the ``Variant``).
         """
-        chess960 = self.headers.get("Variant", "").lower() in [
-            "chess960",
-            "chess 960",
-            "fischerandom",  # Cute Chess
-            "fischerrandom"]
-
-        # http://www.freechess.org/Help/HelpFiles/wild.html
-        wild = self.headers.get("Variant", "").lower() in [
-            "wild/0", "wild/1", "wild/2", "wild/3", "wild/4", "wild/5",
-            "wild/6", "wild/7", "wild/8", "wild/8a"]
-
-        if chess960 or wild or "Variant" not in self.headers:
-            VariantBoard = chess.Board
-        else:
-            from chess.variant import find_variant
-            VariantBoard = find_variant(self.headers["Variant"])
-
-        fen = self.headers.get("FEN", VariantBoard.starting_fen)
-        board = VariantBoard(fen, chess960=chess960)
-        board.chess960 = board.chess960 or board.has_chess960_castling_rights()
-        return board
+        return self.headers.board()
 
     def setup(self, board):
         """
-        Setup a specific starting position. This sets (or resets) the
+        Sets up a specific starting position. This sets (or resets) the
         ``FEN``, ``SetUp``, and ``Variant`` header tags.
         """
         try:
@@ -466,42 +449,34 @@ class Game(GameNode):
     def accept(self, visitor):
         """
         Traverses the game in PGN order using the given *visitor*. Returns
-        the visitor result.
+        the *visitor* result.
         """
-        visitor.begin_game()
+        if visitor.begin_game() is not SKIP:
+            for tagname, tagvalue in self.headers.items():
+                visitor.visit_header(tagname, tagvalue)
+            if visitor.end_headers() is not SKIP:
+                if self.comment:
+                    visitor.visit_comment(self.comment)
 
-        visitor.begin_headers()
-        for tagname, tagvalue in self.headers.items():
-            visitor.visit_header(tagname, tagvalue)
-        visitor.end_headers()
+                if self.variations:
+                    self.variations[0].accept(visitor, _parent_board=self.board())
 
-        if self.comment:
-            visitor.visit_comment(self.comment)
+                visitor.visit_result(self.headers.get("Result", "*"))
 
-        super(Game, self).accept(visitor, _board=self.board())
-
-        visitor.visit_result(self.headers.get("Result", "*"))
         visitor.end_game()
         return visitor.result()
 
     @classmethod
     def from_board(cls, board):
         """Creates a game from the move stack of a :class:`~chess.Board()`."""
-        # Undo all moves.
-        switchyard = collections.deque()
-        while board.move_stack:
-            switchyard.append(board.pop())
-
         # Setup the initial position.
         game = cls()
-        game.setup(board)
+        game.setup(board.root())
         node = game
 
         # Replay all moves.
-        while switchyard:
-            move = switchyard.pop()
+        for move in board.move_stack:
             node = node.add_variation(move)
-            board.push(move)
 
         game.headers["Result"] = board.result()
         return game
@@ -511,8 +486,15 @@ class Game(GameNode):
         """Creates an empty game without the default 7 tag roster."""
         return cls(headers={})
 
+    def __repr__(self):
+        return "<Game at {} ({} vs. {}, {})>".format(
+            hex(id(self)),
+            repr(self.headers.get("White", "?")),
+            repr(self.headers.get("Black", "?")),
+            self.headers.get("Date", "????.??.??"))
 
-class Headers(MutableMapping):
+
+class Headers(collections.abc.MutableMapping):
     def __init__(self, data=None, **kwargs):
         self._tag_roster = {}
         self._others = {}
@@ -530,6 +512,35 @@ class Headers(MutableMapping):
 
         self.update(data, **kwargs)
 
+    def is_chess960(self):
+        return self.get("Variant", "").lower() in [
+            "chess960",
+            "chess 960",
+            "fischerandom",  # Cute Chess
+            "fischerrandom",
+            "fischer random",
+        ]
+
+    def is_wild(self):
+        # http://www.freechess.org/Help/HelpFiles/wild.html
+        return self.get("Variant", "").lower() in [
+            "wild/0", "wild/1", "wild/2", "wild/3", "wild/4", "wild/5",
+            "wild/6", "wild/7", "wild/8", "wild/8a"]
+
+    def variant(self):
+        if "Variant" not in self or self.is_chess960() or self.is_wild():
+            return chess.Board
+        else:
+            from chess.variant import find_variant
+            return find_variant(self["Variant"])
+
+    def board(self):
+        VariantBoard = self.variant()
+        fen = self.get("FEN", VariantBoard.starting_fen)
+        board = VariantBoard(fen, chess960=self.is_chess960())
+        board.chess960 = board.chess960 or board.has_chess960_castling_rights()
+        return board
+
     def __setitem__(self, key, value):
         if key in TAG_ROSTER:
             self._tag_roster[key] = value
@@ -557,8 +568,7 @@ class Headers(MutableMapping):
             if key in self._tag_roster:
                 yield key
 
-        for key in sorted(self._others):
-            yield key
+        yield from sorted(self._others)
 
     def __len__(self):
         return len(self._tag_roster) + len(self._others)
@@ -575,7 +585,68 @@ class Headers(MutableMapping):
             ", ".join("{}={}".format(key, repr(value)) for key, value in self.items()))
 
 
-class BaseVisitor(object):
+class Mainline:
+    def __init__(self, start, f=lambda node: node):
+        self.start = start
+        self.f = f
+
+    def __bool__(self):
+        return bool(self.start.variations)
+
+    def __iter__(self):
+        node = self.start
+        while node.variations:
+            node = node.variations[0]
+            yield self.f(node)
+
+    def __reversed__(self):
+        return ReverseMainline(self.start, self.f)
+
+    def accept(self, visitor):
+        node = self.start
+        board = self.start.board()
+        while node.variations:
+            node = node.variations[0]
+            node._accept_node(board, visitor)
+            board.push(node.move)
+        return visitor.result()
+
+    def __str__(self):
+        return self.accept(StringExporter(columns=None))
+
+    def __repr__(self):
+        return "<Mainline at {} ({})>".format(hex(id(self)), self.accept(StringExporter(columns=None, comments=False)))
+
+
+class ReverseMainline:
+    def __init__(self, stop, f=lambda node: node):
+        self.stop = stop
+        self.f = f
+
+        self.length = 0
+        node = stop
+        while node.variations:
+            node = node.variations[0]
+            self.length += 1
+        self.end = node
+
+    def __len__(self):
+        return self.length
+
+    def __iter__(self):
+        node = self.end
+        while node.parent and node != self.stop:
+            yield self.f(node)
+            node = node.parent
+
+    def __reversed__(self):
+        return Mainline(self.stop, self.f)
+
+    def __repr__(self):
+        return "<ReverseMainline at {} ({})>".format(hex(id(self)), " ".join(ReverseMainline(self.stop, lambda node: node.move.uci())))
+
+
+class BaseVisitor:
     """
     Base class for visitors.
 
@@ -590,7 +661,7 @@ class BaseVisitor(object):
         pass
 
     def begin_headers(self):
-        """Called at the start of game headers."""
+        """Called before visiting game headers."""
         pass
 
     def visit_header(self, tagname, tagvalue):
@@ -598,9 +669,25 @@ class BaseVisitor(object):
         pass
 
     def end_headers(self):
-        """Called at the end of the game headers."""
+        """Called after visiting game headers."""
         pass
 
+    def parse_san(self, board, san):
+        """
+        When the visitor is used by a parser, this is called to parse a move
+        in standard algebraic notation.
+
+        You can override the default implementation to work around specific
+        quirks of your input format.
+        """
+        # Replace zeros with correct castling notation.
+        if san == "0-0":
+            san = "O-O"
+        elif san == "0-0-0":
+            san = "O-O-O"
+
+        return board.parse_san(san)
+
     def visit_move(self, board, move):
         """
         Called for each move.
@@ -621,7 +708,7 @@ class BaseVisitor(object):
     def begin_variation(self):
         """
         Called at the start of a new variation. It is not called for the
-        main line of the game.
+        mainline of the game.
         """
         pass
 
@@ -653,7 +740,7 @@ class GameModelCreator(BaseVisitor):
     Creates a game model. Default visitor for :func:`~chess.pgn.read_game()`.
     """
 
-    def __init__(self):
+    def begin_game(self):
         self.game = Game()
 
         self.variation_stack = [self.game]
@@ -710,6 +797,35 @@ class GameModelCreator(BaseVisitor):
         return self.game
 
 
+class HeaderCreator(BaseVisitor):
+    """Collects headers into a dictionary."""
+
+    def begin_headers(self):
+        self.headers = Headers({})
+
+    def visit_header(self, tagname, tagvalue):
+        self.headers[tagname] = tagvalue
+
+    def end_headers(self):
+        return SKIP
+
+    def result(self):
+        return self.headers
+
+
+class SkipVisitor(BaseVisitor):
+    """Skips a game."""
+
+    def begin_game(self):
+        return SKIP
+
+    def end_headers(self):
+        return SKIP
+
+    def begin_variation(self):
+        return SKIP
+
+
 class StringExporter(BaseVisitor):
     """
     Allows exporting a game as a string.
@@ -721,14 +837,14 @@ class StringExporter(BaseVisitor):
     >>> exporter = chess.pgn.StringExporter(headers=True, variations=True, comments=True)
     >>> pgn_string = game.accept(exporter)
 
-    Only *columns* characters are written per line. If *columns* is ``None``
+    Only *columns* characters are written per line. If *columns* is ``None``,
     then the entire movetext will be on a single line. This does not affect
     header tags and comments.
 
     There will be no newline characters at the end of the string.
     """
 
-    def __init__(self, columns=80, headers=True, comments=True, variations=True):
+    def __init__(self, *, columns=80, headers=True, comments=True, variations=True):
         self.columns = columns
         self.headers = headers
         self.comments = comments
@@ -784,6 +900,8 @@ class StringExporter(BaseVisitor):
         if self.variations:
             self.write_token(") ")
             self.force_movenumber = True
+        else:
+            return SKIP
 
     def visit_comment(self, comment):
         if self.comments and (self.variations or not self.variation_depth):
@@ -837,8 +955,8 @@ class FileExporter(StringExporter):
     >>> game.accept(exporter)
     """
 
-    def __init__(self, handle, columns=80, headers=True, comments=True, variations=True):
-        super(FileExporter, self).__init__(columns=columns, headers=headers, comments=comments, variations=variations)
+    def __init__(self, handle, *, columns=80, headers=True, comments=True, variations=True):
+        super().__init__(columns=columns, headers=headers, comments=comments, variations=variations)
         self.handle = handle
 
     def flush_current_line(self):
@@ -862,7 +980,7 @@ class FileExporter(StringExporter):
         return self.__repr__()
 
 
-def read_game(handle, Visitor=GameModelCreator):
+def read_game(handle, *, Visitor=GameModelCreator):
     """
     Reads a game from a file opened in text mode.
 
@@ -878,7 +996,7 @@ def read_game(handle, Visitor=GameModelCreator):
     >>>
     >>> # Iterate through all moves and play them on a board.
     >>> board = first_game.board()
-    >>> for move in first_game.main_line():
+    >>> for move in first_game.mainline_moves():
     ...     board.push(move)
     ...
     >>> board
@@ -889,19 +1007,13 @@ def read_game(handle, Visitor=GameModelCreator):
     PGN files are ASCII or UTF-8 most of the time. So, the following should
     cover most relevant cases (ASCII, UTF-8, UTF-8 with BOM).
 
-    >>> # Python 3
     >>> pgn = open("data/pgn/kasparov-deep-blue-1997.pgn", encoding="utf-8-sig")
 
     Use :class:`~io.StringIO` to parse games from a string.
 
-    >>> pgn_string = "1. e4 e5 2. Nf3 *"
+    >>> import io
     >>>
-    >>> try:
-    ...     from StringIO import StringIO  # Python 2
-    ... except ImportError:
-    ...     from io import StringIO  # Python 3
-    ...
-    >>> pgn = StringIO(pgn_string)
+    >>> pgn = io.StringIO("1. e4 e5 2. Nf3 *")
     >>> game = chess.pgn.read_game(pgn)
 
     The end of a game is determined by a completely blank line or the end of
@@ -920,8 +1032,9 @@ def read_game(handle, Visitor=GameModelCreator):
     """
     visitor = Visitor()
 
-    dummy_game = Game.without_tag_roster()
     found_game = False
+    skipping_game = False
+    headers = None
 
     # Skip leading empty lines and comments.
     line = handle.readline()
@@ -935,36 +1048,71 @@ def read_game(handle, Visitor=GameModelCreator):
             line = handle.readline()
             continue
 
-        # Read header tags.
-        tag_match = TAG_REGEX.match(line)
-        if tag_match:
-            if not found_game:
-                found_game = True
-                visitor.begin_game()
-                visitor.begin_headers()
+        if not found_game:
+            found_game = True
+            skipping_game = visitor.begin_game() is SKIP
 
-            dummy_game.headers[tag_match.group(1)] = tag_match.group(2)
-            visitor.visit_header(tag_match.group(1), tag_match.group(2))
-        else:
+        if not line.startswith("["):
             break
 
+        if not skipping_game:
+            tag_match = TAG_REGEX.match(line)
+            if tag_match:
+                if headers is None:
+                    headers = Headers({})
+                    visitor.begin_headers()
+
+                headers[tag_match.group(1)] = tag_match.group(2)
+                visitor.visit_header(tag_match.group(1), tag_match.group(2))
+            else:
+                break
+
         line = handle.readline()
 
-    if found_game:
-        visitor.end_headers()
+    if not found_game:
+        return None
+
+    if headers is not None:
+        skipping_game = visitor.end_headers() is SKIP
 
     # Skip a single empty line after headers.
     if line.isspace():
         line = handle.readline()
 
-    # Movetext parser state.
+    # Fast path: Skip entire game.
+    if skipping_game:
+        in_comment = False
+
+        while line:
+            if not in_comment and line.isspace():
+                break
+            elif (not in_comment and "{" in line) or (in_comment and "}" in line):
+                in_comment = line.rfind("{") > line.rfind("}")
+
+            line = handle.readline()
+
+        visitor.end_game()
+        return visitor.result()
+
+    # Chess variant and initial position.
+    if headers is None:
+        headers = Headers({})
+
     try:
-        board_stack = [dummy_game.board()]
+        VariantBoard = headers.variant()
     except ValueError as error:
         visitor.handle_error(error)
-        board_stack = [chess.Board()]
+        VariantBoard = chess.Board
+
+    fen = headers.get("FEN", VariantBoard.starting_fen)
+    try:
+        board_stack = [VariantBoard(fen, chess960=headers.is_chess960())]
+    except ValueError as error:
+        visitor.handle_error(error)
+        board_stack = [VariantBoard(chess960=headers.is_chess960())]
 
     # Parse movetext.
+    skip_variation_depth = 0
     while line:
         read_next_line = True
 
@@ -973,19 +1121,14 @@ def read_game(handle, Visitor=GameModelCreator):
             line = handle.readline()
             continue
 
-        # An empty line means the end of a game. But gracefully try to find
-        # at least some content if we didn't even see headers so far.
-        if found_game and line.isspace():
+        # An empty line means the end of a game.
+        if line.isspace():
             visitor.end_game()
             return visitor.result()
 
         for match in MOVETEXT_REGEX.finditer(line):
             token = match.group(0)
 
-            if not found_game:
-                found_game = True
-                visitor.begin_game()
-
             if token.startswith("{"):
                 # Consume until the end of the comment.
                 line = token[1:]
@@ -1000,12 +1143,32 @@ def read_game(handle, Visitor=GameModelCreator):
                 else:
                     line = ""
 
-                visitor.visit_comment("\n".join(comment_lines).strip())
+                if not skip_variation_depth:
+                    visitor.visit_comment("\n".join(comment_lines).strip())
 
                 # Continue with the current or the next line.
                 if line:
                     read_next_line = False
                 break
+            elif token == "(" and board_stack[-1].move_stack:
+                if skip_variation_depth:
+                    skip_variation_depth += 1
+                elif visitor.begin_variation() is SKIP:
+                    skip_variation_depth = 1
+                else:
+                    board = board_stack[-1].copy()
+                    board.pop()
+                    board_stack.append(board)
+            elif token == ")" and skip_variation_depth:
+                skip_variation_depth -= 1
+                if not skip_variation_depth:
+                    visitor.end_variation()
+            elif token == ")" and len(board_stack) > 1:
+                # Always leave at least the root node on the stack.
+                visitor.end_variation()
+                board_stack.pop()
+            elif skip_variation_depth:
+                continue
             elif token.startswith(";"):
                 break
             elif token.startswith("$"):
@@ -1028,28 +1191,12 @@ def read_game(handle, Visitor=GameModelCreator):
                 visitor.visit_nag(NAG_SPECULATIVE_MOVE)
             elif token == "?!":
                 visitor.visit_nag(NAG_DUBIOUS_MOVE)
-            elif token == "(" and board_stack[-1].move_stack:
-                visitor.begin_variation()
-
-                board = board_stack[-1].copy()
-                board.pop()
-                board_stack.append(board)
-            elif token == ")" and len(board_stack) > 1:
-                # Always leave at least the root node on the stack.
-                visitor.end_variation()
-                board_stack.pop()
             elif token in ["1-0", "0-1", "1/2-1/2", "*"] and len(board_stack) == 1:
                 visitor.visit_result(token)
             else:
-                # Replace zeros castling notation.
-                if token == "0-0":
-                    token = "O-O"
-                elif token == "0-0-0":
-                    token = "O-O-O"
-
                 # Parse SAN tokens.
                 try:
-                    move = board_stack[-1].parse_san(token)
+                    move = visitor.parse_san(board_stack[-1], token)
                 except ValueError as error:
                     visitor.handle_error(error)
                 else:
@@ -1059,17 +1206,13 @@ def read_game(handle, Visitor=GameModelCreator):
         if read_next_line:
             line = handle.readline()
 
-    if found_game:
-        visitor.end_game()
-        return visitor.result()
+    visitor.end_game()
+    return visitor.result()
 
 
-def scan_headers(handle):
+def read_headers(handle):
     """
-    Scan a PGN file opened in text mode for game offsets and headers.
-
-    Yields a tuple for each game. The first element is the offset and the
-    second element is a mapping of game headers.
+    Reads game headers from a PGN file opened in text mode.
 
     Since actually parsing many games from a big file is relatively expensive,
     this is a better way to look only for specific games and then seek and
@@ -1081,97 +1224,35 @@ def scan_headers(handle):
     >>>
     >>> pgn = open("data/pgn/kasparov-deep-blue-1997.pgn")
     >>>
-    >>> for offset, headers in chess.pgn.scan_headers(pgn):
-    ...     if "Kasparov" in headers["White"]:
-    ...         kasparov_offset = offset
+    >>> kasparov_offsets = []
+    >>>
+    >>> while True:
+    ...     offset = pgn.tell()
+    ...
+    ...     headers = chess.pgn.read_headers(pgn)
+    ...     if headers is None:
     ...         break
+    ...
+    ...     if "Kasparov" in headers.get("White", "?"):
+    ...         kasparov_offsets.append(offset)
 
     Then it can later be seeked an parsed.
 
-    >>> pgn.seek(kasparov_offset)
+    >>> for offset in kasparov_offsets:
+    ...     pgn.seek(offset)
+    ...     chess.pgn.read_game(pgn)  # doctest: +ELLIPSIS
     0
-    >>> game = chess.pgn.read_game(pgn)
-
-    This also works nicely with generators, scanning lazily only when the next
-    offset is required.
-
-    >>> white_win_offsets = (offset for offset, headers in chess.pgn.scan_headers(pgn)
-    ...                             if headers["Result"] == "1-0")
-    >>> first_white_win = next(white_win_offsets)
-    >>> second_white_win = next(white_win_offsets)
-
-    :warning: Be careful when seeking a game in the file while more offsets are
-        being generated.
+    <Game at ... ('Garry Kasparov' vs. 'Deep Blue (Computer)', 1997.??.??)>
+    1436
+    <Game at ... ('Garry Kasparov' vs. 'Deep Blue (Computer)', 1997.??.??)>
+    3067
+    <Game at ... ('Garry Kasparov' vs. 'Deep Blue (Computer)', 1997.??.??)>
     """
-    in_comment = False
-
-    game_headers = None
-    game_pos = None
-
-    last_pos = handle.tell()
-    line = handle.readline()
-
-    while line:
-        # Skip single-line comments.
-        if line.startswith("%"):
-            last_pos = handle.tell()
-            line = handle.readline()
-            continue
-
-        # Reading a header tag. Parse it and add it to the current headers.
-        if not in_comment and line.startswith("["):
-            tag_match = TAG_REGEX.match(line)
-            if tag_match:
-                if game_pos is None:
-                    game_headers = Headers()
-                    game_pos = last_pos
-
-                game_headers[tag_match.group(1)] = tag_match.group(2)
-
-                last_pos = handle.tell()
-                line = handle.readline()
-                continue
-
-        # Reading movetext. Update parser's in_comment state in order to skip
-        # comments that look like header tags.
-        if (not in_comment and "{" in line) or (in_comment and "}" in line):
-            in_comment = line.rfind("{") > line.rfind("}")
-
-        # Reading movetext. If there were headers previously, those are now
-        # complete and can be yielded.
-        if game_pos is not None:
-            yield game_pos, game_headers
-            game_pos = None
-
-        last_pos = handle.tell()
-        line = handle.readline()
-
-    # Yield the headers of the last game.
-    if game_pos is not None:
-        yield game_pos, game_headers
+    return read_game(handle, Visitor=HeaderCreator)
 
 
-def scan_offsets(handle):
+def skip_game(handle):
     """
-    Scan a PGN file opened in text mode for game offsets.
-
-    Yields the starting offsets of all the games, so that they can be seeked
-    later. This is just like :func:`~chess.pgn.scan_headers()` but more
-    efficient if you do not actually need the header information.
-
-    The PGN standard requires each game to start with an ``Event`` tag. So does
-    this scanner.
+    Skip a game. Returns ``True`` if a game was found and skipped.
     """
-    in_comment = False
-
-    last_pos = handle.tell()
-    line = handle.readline()
-
-    while line:
-        if not in_comment and line.startswith("[Event \""):
-            yield last_pos
-        elif (not in_comment and "{" in line) or (in_comment and "}" in line):
-            in_comment = line.rfind("{") > line.rfind("}")
-
-        last_pos = handle.tell()
-        line = handle.readline()
+    return read_game(handle, Visitor=SkipVisitor)
